Package: httptest2
Type: Package
Title: Test Helpers for 'httr2'
Description: Testing and documenting code that communicates with remote servers
    can be painful. This package helps with writing tests for packages that
    use 'httr2'. It enables testing all of the logic
    on the R sides of the API without requiring access to the
    remote service, and it also allows recording real API responses to use as
    test fixtures. The ability to save responses and load them offline also
    enables writing vignettes and other dynamic documents that can be
    distributed without access to a live server.
Version: 1.2.2
Authors@R: c(
        person("Neal", "Richardson", role=c("aut", "cre"), email="neal.p.richardson@gmail.com", comment=c(ORCID="0009-0002-7992-3520")),
        person("Jonathan", "Keane", role="ctb", email="jkeane@gmail.com"),
        person("Maëlle", "Salmon", role="ctb", email="maelle.salmon@yahoo.se", comment=c(ORCID="0000-0002-2815-0399"))
    )
URL: https://enpiar.com/httptest2/,
        https://github.com/nealrichardson/httptest2
BugReports: https://github.com/nealrichardson/httptest2/issues
License: MIT + file LICENSE
Imports: digest, httr2 (>= 0.2.0), jsonlite, rlang, stats, testthat (>=
        3.1.5), utils
Suggests: curl, knitr, pkgload, rmarkdown, spelling, webfakes, xml2
Language: en-US
RoxygenNote: 7.3.3
Encoding: UTF-8
VignetteBuilder: knitr
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2025-11-16 19:42:32 UTC; npr
Author: Neal Richardson [aut, cre] (ORCID:
    <https://orcid.org/0009-0002-7992-3520>),
  Jonathan Keane [ctb],
  Maëlle Salmon [ctb] (ORCID: <https://orcid.org/0000-0002-2815-0399>)
Maintainer: Neal Richardson <neal.p.richardson@gmail.com>
Repository: CRAN
Date/Publication: 2025-11-16 20:10:02 UTC
Built: R 4.6.0; ; 2026-04-28 18:22:04 UTC; unix
