# Generated by roxygen2: do not edit by hand

S3method(cnd_body,httptest2_request)
S3method(cnd_footer,httptest2_request)
S3method(cnd_header,httptest2_request)
export(.mockPaths)
export(block_requests)
export(build_mock_url)
export(capture_requests)
export(change_state)
export(end_vignette)
export(expect_DELETE)
export(expect_GET)
export(expect_PATCH)
export(expect_POST)
export(expect_PUT)
export(expect_no_request)
export(expect_request_header)
export(find_mock_file)
export(get_current_redactor)
export(gsub_response)
export(redact_cookies)
export(redact_headers)
export(save_response)
export(set_redactor)
export(start_capturing)
export(start_vignette)
export(stop_capturing)
export(stop_mocking)
export(use_httptest2)
export(use_mock_api)
export(with_mock_api)
export(with_mock_dir)
export(with_trace)
export(within_body_text)
export(without_internet)
importFrom(digest,digest)
importFrom(httr2,req_perform)
importFrom(httr2,request)
importFrom(httr2,resp_body_string)
importFrom(httr2,resp_content_type)
importFrom(httr2,resp_status)
importFrom(httr2,response)
importFrom(jsonlite,prettify)
importFrom(rlang,abort)
importFrom(rlang,cnd_body)
importFrom(rlang,cnd_footer)
importFrom(rlang,cnd_header)
importFrom(rlang,is_string)
importFrom(stats,terms)
importFrom(testthat,expect_error)
importFrom(testthat,expect_match)
importFrom(testthat,expect_no_error)
importFrom(testthat,expect_null)
importFrom(testthat,fail)
importFrom(utils,sessionInfo)
importFrom(utils,tail)
